# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Session Service (2.15.4)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from .utils import randomize

from ..api.session.models import ApimodelsAppendTeamGameSessionRequest
from ..api.session.models import ApimodelsConfigAlertRequestCreate
from ..api.session.models import ApimodelsConfigAlertResponse
from ..api.session.models import ApimodelsConfigurationTemplateResponse
from ..api.session.models import ApimodelsConfigurationTemplatesResponse
from ..api.session.models import ApimodelsCreateConfigurationTemplateRequest
from ..api.session.models import ApimodelsCreateGameSessionRequest
from ..api.session.models import ApimodelsCreatePartyRequest
from ..api.session.models import ApimodelsDSInformationResponse
from ..api.session.models import ApimodelsDeleteBulkGameSessionRequest
from ..api.session.models import ApimodelsDeleteBulkGameSessionsAPIResponse
from ..api.session.models import ApimodelsGameSessionQueryResponse
from ..api.session.models import ApimodelsGameSessionResponse
from ..api.session.models import ApimodelsJoinByCodeRequest
from ..api.session.models import ApimodelsKickResponse
from ..api.session.models import ApimodelsPagination
from ..api.session.models import ApimodelsPartyQueryResponse
from ..api.session.models import ApimodelsPartySessionResponse
from ..api.session.models import ApimodelsPlayerAttributesRequestBody
from ..api.session.models import ApimodelsPlayerAttributesResponseBody
from ..api.session.models import ApimodelsPlayerPlatform
from ..api.session.models import ApimodelsPlayersCurrentPlatformRequest
from ..api.session.models import ApimodelsPlayersCurrentPlatformResponse
from ..api.session.models import ApimodelsPromoteLeaderRequest
from ..api.session.models import ApimodelsPublicConfiguration
from ..api.session.models import ApimodelsRequestMember
from ..api.session.models import ApimodelsResponseDeleteBulkGameSessions
from ..api.session.models import ApimodelsSessionInviteRequest
from ..api.session.models import ApimodelsUpdateConfigurationTemplateRequest
from ..api.session.models import ApimodelsUpdateGameSessionBackfillRequest
from ..api.session.models import ApimodelsUpdateGameSessionMemberStatusResponse
from ..api.session.models import ApimodelsUpdateGameSessionRequest
from ..api.session.models import ApimodelsUpdatePartyRequest
from ..api.session.models import ApimodelsUserResponse
from ..api.session.models import ModelsDSMConfigRecord
from ..api.session.models import ModelsDefaultDSMCConfig
from ..api.session.models import ModelsGameServer
from ..api.session.models import ModelsNativeSessionSetting
from ..api.session.models import ModelsPartyMembers
from ..api.session.models import ModelsTeam
from ..api.session.models import ModelsUserPlatformInfo
from ..api.session.models import ResponseError


def create_apimodels_append_team_game_session_request_example() -> (
    ApimodelsAppendTeamGameSessionRequest
):
    instance = ApimodelsAppendTeamGameSessionRequest()
    instance.additional_members = [create_models_party_members_example()]
    instance.proposed_teams = [create_models_team_example()]
    instance.version = randomize("int", min_val=1, max_val=1000)
    return instance


def create_apimodels_config_alert_request_create_example() -> (
    ApimodelsConfigAlertRequestCreate
):
    instance = ApimodelsConfigAlertRequestCreate()
    instance.duration_days = randomize("int", min_val=1, max_val=1000)
    return instance


def create_apimodels_config_alert_response_example() -> ApimodelsConfigAlertResponse:
    instance = ApimodelsConfigAlertResponse()
    instance.created_at = randomize("date")
    instance.duration_days = randomize("int", min_val=1, max_val=1000)
    instance.namespace = randomize("slug")
    instance.updated_at = randomize("date")
    return instance


def create_apimodels_configuration_template_response_example() -> (
    ApimodelsConfigurationTemplateResponse
):
    instance = ApimodelsConfigurationTemplateResponse()
    instance.client_version = randomize()
    instance.created_at = randomize()
    instance.deployment = randomize()
    instance.inactive_timeout = randomize("int", min_val=1, max_val=1000)
    instance.invite_timeout = randomize("int", min_val=1, max_val=1000)
    instance.joinability = randomize()
    instance.last = randomize()
    instance.max_players = randomize("int", min_val=1, max_val=1000)
    instance.min_players = randomize("int", min_val=1, max_val=1000)
    instance.name = randomize()
    instance.namespace = randomize("slug")
    instance.persistent = randomize("bool")
    instance.text_chat = randomize("bool")
    instance.type_ = randomize()
    instance.updated_at = randomize()
    instance.ds_source = randomize()
    instance.fallback_claim_keys = [randomize()]
    instance.max_active_sessions = randomize("int", min_val=1, max_val=1000)
    instance.native_session_setting = create_models_native_session_setting_example()
    instance.preferred_claim_keys = [randomize()]
    instance.requested_regions = [randomize()]
    return instance


def create_apimodels_configuration_templates_response_example() -> (
    ApimodelsConfigurationTemplatesResponse
):
    instance = ApimodelsConfigurationTemplatesResponse()
    instance.data = [create_apimodels_configuration_template_response_example()]
    instance.paging = create_apimodels_pagination_example()
    return instance


def create_apimodels_create_configuration_template_request_example() -> (
    ApimodelsCreateConfigurationTemplateRequest
):
    instance = ApimodelsCreateConfigurationTemplateRequest()
    instance.client_version = randomize()
    instance.deployment = randomize()
    instance.inactive_timeout = randomize("int", min_val=1, max_val=1000)
    instance.invite_timeout = randomize("int", min_val=1, max_val=1000)
    instance.joinability = randomize()
    instance.max_players = randomize("int", min_val=1, max_val=1000)
    instance.min_players = randomize("int", min_val=1, max_val=1000)
    instance.name = randomize()
    instance.persistent = randomize("bool")
    instance.requested_regions = [randomize()]
    instance.text_chat = randomize("bool")
    instance.type_ = randomize()
    instance.ds_source = randomize()
    instance.fallback_claim_keys = [randomize()]
    instance.max_active_sessions = randomize("int", min_val=1, max_val=1000)
    instance.native_session_setting = create_models_native_session_setting_example()
    instance.preferred_claim_keys = [randomize()]
    return instance


def create_apimodels_create_game_session_request_example() -> (
    ApimodelsCreateGameSessionRequest
):
    instance = ApimodelsCreateGameSessionRequest()
    instance.attributes = {randomize(): randomize()}
    instance.backfill_ticket_id = randomize()
    instance.client_version = randomize()
    instance.configuration_name = randomize()
    instance.deployment = randomize()
    instance.inactive_timeout = randomize("int", min_val=1, max_val=1000)
    instance.invite_timeout = randomize("int", min_val=1, max_val=1000)
    instance.joinability = randomize()
    instance.match_pool = randomize()
    instance.max_players = randomize("int", min_val=1, max_val=1000)
    instance.min_players = randomize("int", min_val=1, max_val=1000)
    instance.requested_regions = [randomize()]
    instance.server_name = randomize()
    instance.teams = [create_models_team_example()]
    instance.text_chat = randomize("bool")
    instance.ticket_i_ds = [randomize()]
    instance.type_ = randomize()
    instance.ds_source = randomize()
    instance.fallback_claim_keys = [randomize()]
    instance.preferred_claim_keys = [randomize()]
    return instance


def create_apimodels_create_party_request_example() -> ApimodelsCreatePartyRequest:
    instance = ApimodelsCreatePartyRequest()
    instance.attributes = {randomize(): randomize()}
    instance.configuration_name = randomize()
    instance.inactive_timeout = randomize("int", min_val=1, max_val=1000)
    instance.invite_timeout = randomize("int", min_val=1, max_val=1000)
    instance.joinability = randomize()
    instance.max_players = randomize("int", min_val=1, max_val=1000)
    instance.members = [create_apimodels_request_member_example()]
    instance.min_players = randomize("int", min_val=1, max_val=1000)
    instance.text_chat = randomize("bool")
    instance.type_ = randomize()
    return instance


def create_apimodels_delete_bulk_game_session_request_example() -> (
    ApimodelsDeleteBulkGameSessionRequest
):
    instance = ApimodelsDeleteBulkGameSessionRequest()
    instance.ids = [randomize()]
    return instance


def create_apimodels_delete_bulk_game_sessions_api_response_example() -> (
    ApimodelsDeleteBulkGameSessionsAPIResponse
):
    instance = ApimodelsDeleteBulkGameSessionsAPIResponse()
    instance.failed = [create_apimodels_response_delete_bulk_game_sessions_example()]
    instance.success = [randomize()]
    return instance


def create_apimodels_ds_information_response_example() -> (
    ApimodelsDSInformationResponse
):
    instance = ApimodelsDSInformationResponse()
    instance.requested_at = randomize()
    instance.status = randomize()
    instance.status_v2 = randomize()
    instance.server = create_models_game_server_example()
    return instance


def create_apimodels_game_session_query_response_example() -> (
    ApimodelsGameSessionQueryResponse
):
    instance = ApimodelsGameSessionQueryResponse()
    instance.data = [create_apimodels_game_session_response_example()]
    instance.paging = create_apimodels_pagination_example()
    return instance


def create_apimodels_game_session_response_example() -> ApimodelsGameSessionResponse:
    instance = ApimodelsGameSessionResponse()
    instance.backfill_ticket_id = randomize()
    instance.configuration = create_apimodels_public_configuration_example()
    instance.created_at = randomize()
    instance.created_by = randomize()
    instance.ds_information = create_apimodels_ds_information_response_example()
    instance.id_ = randomize()
    instance.is_active = randomize("bool")
    instance.is_full = randomize("bool")
    instance.leader_id = randomize()
    instance.match_pool = randomize()
    instance.members = [create_apimodels_user_response_example()]
    instance.namespace = randomize("slug")
    instance.updated_at = randomize()
    instance.version = randomize("int", min_val=1, max_val=1000)
    instance.attributes = {randomize(): randomize()}
    instance.code = randomize()
    instance.expired_at = randomize()
    instance.teams = [create_models_team_example()]
    instance.ticket_i_ds = [randomize()]
    return instance


def create_apimodels_join_by_code_request_example() -> ApimodelsJoinByCodeRequest:
    instance = ApimodelsJoinByCodeRequest()
    instance.code = randomize()
    return instance


def create_apimodels_kick_response_example() -> ApimodelsKickResponse:
    instance = ApimodelsKickResponse()
    instance.leader_id = randomize()
    instance.members = [create_apimodels_user_response_example()]
    instance.party_id = randomize("uid")
    return instance


def create_apimodels_pagination_example() -> ApimodelsPagination:
    instance = ApimodelsPagination()
    instance.first = randomize()
    instance.last = randomize()
    instance.next_ = randomize()
    instance.previous = randomize()
    return instance


def create_apimodels_party_query_response_example() -> ApimodelsPartyQueryResponse:
    instance = ApimodelsPartyQueryResponse()
    instance.data = [create_apimodels_party_session_response_example()]
    instance.paging = create_apimodels_pagination_example()
    return instance


def create_apimodels_party_session_response_example() -> ApimodelsPartySessionResponse:
    instance = ApimodelsPartySessionResponse()
    instance.configuration = create_apimodels_public_configuration_example()
    instance.created_at = randomize()
    instance.created_by = randomize()
    instance.id_ = randomize()
    instance.is_active = randomize("bool")
    instance.is_full = randomize("bool")
    instance.leader_id = randomize()
    instance.members = [create_apimodels_user_response_example()]
    instance.namespace = randomize("slug")
    instance.updated_at = randomize()
    instance.version = randomize("int", min_val=1, max_val=1000)
    instance.attributes = {randomize(): randomize()}
    instance.code = randomize()
    instance.expired_at = randomize()
    return instance


def create_apimodels_player_attributes_request_body_example() -> (
    ApimodelsPlayerAttributesRequestBody
):
    instance = ApimodelsPlayerAttributesRequestBody()
    instance.crossplay_enabled = randomize("bool")
    instance.current_platform = randomize()
    instance.data = {randomize(): randomize()}
    instance.platforms = [create_models_user_platform_info_example()]
    instance.roles = [randomize()]
    return instance


def create_apimodels_player_attributes_response_body_example() -> (
    ApimodelsPlayerAttributesResponseBody
):
    instance = ApimodelsPlayerAttributesResponseBody()
    instance.crossplay_enabled = randomize("bool")
    instance.current_platform = randomize()
    instance.data = {randomize(): randomize()}
    instance.platforms = [create_models_user_platform_info_example()]
    instance.roles = [randomize()]
    instance.user_id = randomize("uid")
    return instance


def create_apimodels_player_platform_example() -> ApimodelsPlayerPlatform:
    instance = ApimodelsPlayerPlatform()
    instance.current_platform = randomize()
    instance.user_id = randomize("uid")
    return instance


def create_apimodels_players_current_platform_request_example() -> (
    ApimodelsPlayersCurrentPlatformRequest
):
    instance = ApimodelsPlayersCurrentPlatformRequest()
    instance.user_i_ds = [randomize()]
    return instance


def create_apimodels_players_current_platform_response_example() -> (
    ApimodelsPlayersCurrentPlatformResponse
):
    instance = ApimodelsPlayersCurrentPlatformResponse()
    instance.data = [create_apimodels_player_platform_example()]
    return instance


def create_apimodels_promote_leader_request_example() -> ApimodelsPromoteLeaderRequest:
    instance = ApimodelsPromoteLeaderRequest()
    instance.leader_id = randomize()
    return instance


def create_apimodels_public_configuration_example() -> ApimodelsPublicConfiguration:
    instance = ApimodelsPublicConfiguration()
    instance.client_version = randomize()
    instance.deployment = randomize()
    instance.inactive_timeout = randomize("int", min_val=1, max_val=1000)
    instance.invite_timeout = randomize("int", min_val=1, max_val=1000)
    instance.joinability = randomize()
    instance.max_players = randomize("int", min_val=1, max_val=1000)
    instance.min_players = randomize("int", min_val=1, max_val=1000)
    instance.name = randomize()
    instance.persistent = randomize("bool")
    instance.text_chat = randomize("bool")
    instance.type_ = randomize()
    instance.ds_source = randomize()
    instance.fallback_claim_keys = [randomize()]
    instance.preferred_claim_keys = [randomize()]
    instance.requested_regions = [randomize()]
    return instance


def create_apimodels_request_member_example() -> ApimodelsRequestMember:
    instance = ApimodelsRequestMember()
    instance.id_ = randomize()
    instance.platform_id = randomize()
    instance.platform_user_id = randomize()
    return instance


def create_apimodels_response_delete_bulk_game_sessions_example() -> (
    ApimodelsResponseDeleteBulkGameSessions
):
    instance = ApimodelsResponseDeleteBulkGameSessions()
    instance.error = randomize()
    instance.id_ = randomize()
    return instance


def create_apimodels_session_invite_request_example() -> ApimodelsSessionInviteRequest:
    instance = ApimodelsSessionInviteRequest()
    instance.platform_id = randomize()
    instance.user_id = randomize("uid")
    return instance


def create_apimodels_update_configuration_template_request_example() -> (
    ApimodelsUpdateConfigurationTemplateRequest
):
    instance = ApimodelsUpdateConfigurationTemplateRequest()
    instance.client_version = randomize()
    instance.deployment = randomize()
    instance.inactive_timeout = randomize("int", min_val=1, max_val=1000)
    instance.invite_timeout = randomize("int", min_val=1, max_val=1000)
    instance.joinability = randomize()
    instance.max_players = randomize("int", min_val=1, max_val=1000)
    instance.min_players = randomize("int", min_val=1, max_val=1000)
    instance.name = randomize()
    instance.persistent = randomize("bool")
    instance.requested_regions = [randomize()]
    instance.text_chat = randomize("bool")
    instance.type_ = randomize()
    instance.ds_source = randomize()
    instance.fallback_claim_keys = [randomize()]
    instance.max_active_sessions = randomize("int", min_val=1, max_val=1000)
    instance.native_session_setting = create_models_native_session_setting_example()
    instance.preferred_claim_keys = [randomize()]
    return instance


def create_apimodels_update_game_session_backfill_request_example() -> (
    ApimodelsUpdateGameSessionBackfillRequest
):
    instance = ApimodelsUpdateGameSessionBackfillRequest()
    instance.backfill_ticket_id = randomize()
    return instance


def create_apimodels_update_game_session_member_status_response_example() -> (
    ApimodelsUpdateGameSessionMemberStatusResponse
):
    instance = ApimodelsUpdateGameSessionMemberStatusResponse()
    instance.status = randomize()
    instance.status_v2 = randomize()
    return instance


def create_apimodels_update_game_session_request_example() -> (
    ApimodelsUpdateGameSessionRequest
):
    instance = ApimodelsUpdateGameSessionRequest()
    instance.attributes = {randomize(): randomize()}
    instance.backfill_ticket_id = randomize()
    instance.client_version = randomize()
    instance.deployment = randomize()
    instance.inactive_timeout = randomize("int", min_val=1, max_val=1000)
    instance.invite_timeout = randomize("int", min_val=1, max_val=1000)
    instance.joinability = randomize()
    instance.match_pool = randomize()
    instance.max_players = randomize("int", min_val=1, max_val=1000)
    instance.min_players = randomize("int", min_val=1, max_val=1000)
    instance.requested_regions = [randomize()]
    instance.teams = [create_models_team_example()]
    instance.ticket_i_ds = [randomize()]
    instance.type_ = randomize()
    instance.version = randomize("int", min_val=1, max_val=1000)
    instance.fallback_claim_keys = [randomize()]
    instance.preferred_claim_keys = [randomize()]
    return instance


def create_apimodels_update_party_request_example() -> ApimodelsUpdatePartyRequest:
    instance = ApimodelsUpdatePartyRequest()
    instance.attributes = {randomize(): randomize()}
    instance.inactive_timeout = randomize("int", min_val=1, max_val=1000)
    instance.invite_timeout = randomize("int", min_val=1, max_val=1000)
    instance.joinability = randomize()
    instance.max_players = randomize("int", min_val=1, max_val=1000)
    instance.min_players = randomize("int", min_val=1, max_val=1000)
    instance.type_ = randomize()
    instance.version = randomize("int", min_val=1, max_val=1000)
    return instance


def create_apimodels_user_response_example() -> ApimodelsUserResponse:
    instance = ApimodelsUserResponse()
    instance.id_ = randomize()
    instance.platform_id = randomize()
    instance.platform_user_id = randomize()
    instance.status = randomize()
    instance.status_v2 = randomize()
    instance.updated_at = randomize()
    return instance


def create_models_default_dsmc_config_example() -> ModelsDefaultDSMCConfig:
    instance = ModelsDefaultDSMCConfig()
    instance.claim_timeout = randomize("int", min_val=1, max_val=1000)
    instance.creation_timeout = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_dsm_config_record_example() -> ModelsDSMConfigRecord:
    instance = ModelsDSMConfigRecord()
    instance.claim_timeout = randomize("int", min_val=1, max_val=1000)
    instance.created_at = randomize("date")
    instance.creation_timeout = randomize("int", min_val=1, max_val=1000)
    instance.namespace = randomize("slug")
    instance.updated_at = randomize("date")
    return instance


def create_models_game_server_example() -> ModelsGameServer:
    instance = ModelsGameServer()
    instance.custom_attribute = randomize()
    instance.deployment = randomize()
    instance.game_version = randomize("version")
    instance.image_version = randomize()
    instance.ip = randomize()
    instance.is_override_game_version = randomize("bool")
    instance.last_update = randomize("date")
    instance.namespace = randomize("slug")
    instance.pod_name = randomize()
    instance.port = randomize("int", min_val=1, max_val=1000)
    instance.ports = {}
    instance.protocol = randomize()
    instance.provider = randomize()
    instance.region = randomize()
    instance.session_id = randomize("uid")
    instance.source = randomize()
    instance.status = randomize()
    instance.alternate_ips = [randomize()]
    return instance


def create_models_native_session_setting_example() -> ModelsNativeSessionSetting:
    instance = ModelsNativeSessionSetting()
    instance.psn_service_label = randomize("int", min_val=1, max_val=1000)
    instance.session_title = randomize()
    instance.xbox_service_config_id = randomize()
    instance.xbox_session_template_name = randomize()
    instance.psn_supported_platforms = [randomize()]
    instance.should_sync = randomize("bool")
    return instance


def create_models_party_members_example() -> ModelsPartyMembers:
    instance = ModelsPartyMembers()
    instance.party_id = randomize("uid")
    instance.user_i_ds = [randomize()]
    return instance


def create_models_team_example() -> ModelsTeam:
    instance = ModelsTeam()
    instance.user_i_ds = [randomize()]
    instance.parties = [create_models_party_members_example()]
    return instance


def create_models_user_platform_info_example() -> ModelsUserPlatformInfo:
    instance = ModelsUserPlatformInfo()
    instance.name = randomize()
    instance.user_id = randomize("uid")
    return instance


def create_response_error_example() -> ResponseError:
    instance = ResponseError()
    instance.attributes = {randomize(): randomize()}
    instance.error_code = randomize("int", min_val=1, max_val=1000)
    instance.error_message = randomize()
    instance.message = randomize()
    instance.name = randomize()
    return instance



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Legal Service (1.29.2)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class UpdateBasePolicyResponse(Model):
    """Update base policy response (UpdateBasePolicyResponse)

    Properties:
        id_: (id) REQUIRED str

        affected_client_ids: (affectedClientIds) OPTIONAL List[str]

        affected_countries: (affectedCountries) OPTIONAL List[str]

        created_at: (createdAt) OPTIONAL str

        description: (description) OPTIONAL str

        global_policy_name: (globalPolicyName) OPTIONAL str

        namespace: (namespace) OPTIONAL str

        policy_id: (policyId) OPTIONAL str

        tags: (tags) OPTIONAL List[str]

        type_id: (typeId) OPTIONAL str

        updated_at: (updatedAt) OPTIONAL str
    """

    # region fields

    id_: str  # REQUIRED
    affected_client_ids: List[str]  # OPTIONAL
    affected_countries: List[str]  # OPTIONAL
    created_at: str  # OPTIONAL
    description: str  # OPTIONAL
    global_policy_name: str  # OPTIONAL
    namespace: str  # OPTIONAL
    policy_id: str  # OPTIONAL
    tags: List[str]  # OPTIONAL
    type_id: str  # OPTIONAL
    updated_at: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_id(self, value: str) -> UpdateBasePolicyResponse:
        self.id_ = value
        return self

    def with_affected_client_ids(self, value: List[str]) -> UpdateBasePolicyResponse:
        self.affected_client_ids = value
        return self

    def with_affected_countries(self, value: List[str]) -> UpdateBasePolicyResponse:
        self.affected_countries = value
        return self

    def with_created_at(self, value: str) -> UpdateBasePolicyResponse:
        self.created_at = value
        return self

    def with_description(self, value: str) -> UpdateBasePolicyResponse:
        self.description = value
        return self

    def with_global_policy_name(self, value: str) -> UpdateBasePolicyResponse:
        self.global_policy_name = value
        return self

    def with_namespace(self, value: str) -> UpdateBasePolicyResponse:
        self.namespace = value
        return self

    def with_policy_id(self, value: str) -> UpdateBasePolicyResponse:
        self.policy_id = value
        return self

    def with_tags(self, value: List[str]) -> UpdateBasePolicyResponse:
        self.tags = value
        return self

    def with_type_id(self, value: str) -> UpdateBasePolicyResponse:
        self.type_id = value
        return self

    def with_updated_at(self, value: str) -> UpdateBasePolicyResponse:
        self.updated_at = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "affected_client_ids"):
            result["affectedClientIds"] = [str(i0) for i0 in self.affected_client_ids]
        elif include_empty:
            result["affectedClientIds"] = []
        if hasattr(self, "affected_countries"):
            result["affectedCountries"] = [str(i0) for i0 in self.affected_countries]
        elif include_empty:
            result["affectedCountries"] = []
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "description"):
            result["description"] = str(self.description)
        elif include_empty:
            result["description"] = ""
        if hasattr(self, "global_policy_name"):
            result["globalPolicyName"] = str(self.global_policy_name)
        elif include_empty:
            result["globalPolicyName"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "policy_id"):
            result["policyId"] = str(self.policy_id)
        elif include_empty:
            result["policyId"] = ""
        if hasattr(self, "tags"):
            result["tags"] = [str(i0) for i0 in self.tags]
        elif include_empty:
            result["tags"] = []
        if hasattr(self, "type_id"):
            result["typeId"] = str(self.type_id)
        elif include_empty:
            result["typeId"] = ""
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        id_: str,
        affected_client_ids: Optional[List[str]] = None,
        affected_countries: Optional[List[str]] = None,
        created_at: Optional[str] = None,
        description: Optional[str] = None,
        global_policy_name: Optional[str] = None,
        namespace: Optional[str] = None,
        policy_id: Optional[str] = None,
        tags: Optional[List[str]] = None,
        type_id: Optional[str] = None,
        updated_at: Optional[str] = None,
        **kwargs,
    ) -> UpdateBasePolicyResponse:
        instance = cls()
        instance.id_ = id_
        if affected_client_ids is not None:
            instance.affected_client_ids = affected_client_ids
        if affected_countries is not None:
            instance.affected_countries = affected_countries
        if created_at is not None:
            instance.created_at = created_at
        if description is not None:
            instance.description = description
        if global_policy_name is not None:
            instance.global_policy_name = global_policy_name
        if namespace is not None:
            instance.namespace = namespace
        if policy_id is not None:
            instance.policy_id = policy_id
        if tags is not None:
            instance.tags = tags
        if type_id is not None:
            instance.type_id = type_id
        if updated_at is not None:
            instance.updated_at = updated_at
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> UpdateBasePolicyResponse:
        instance = cls()
        if not dict_:
            return instance
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "affectedClientIds" in dict_ and dict_["affectedClientIds"] is not None:
            instance.affected_client_ids = [
                str(i0) for i0 in dict_["affectedClientIds"]
            ]
        elif include_empty:
            instance.affected_client_ids = []
        if "affectedCountries" in dict_ and dict_["affectedCountries"] is not None:
            instance.affected_countries = [str(i0) for i0 in dict_["affectedCountries"]]
        elif include_empty:
            instance.affected_countries = []
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "description" in dict_ and dict_["description"] is not None:
            instance.description = str(dict_["description"])
        elif include_empty:
            instance.description = ""
        if "globalPolicyName" in dict_ and dict_["globalPolicyName"] is not None:
            instance.global_policy_name = str(dict_["globalPolicyName"])
        elif include_empty:
            instance.global_policy_name = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "policyId" in dict_ and dict_["policyId"] is not None:
            instance.policy_id = str(dict_["policyId"])
        elif include_empty:
            instance.policy_id = ""
        if "tags" in dict_ and dict_["tags"] is not None:
            instance.tags = [str(i0) for i0 in dict_["tags"]]
        elif include_empty:
            instance.tags = []
        if "typeId" in dict_ and dict_["typeId"] is not None:
            instance.type_id = str(dict_["typeId"])
        elif include_empty:
            instance.type_id = ""
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, UpdateBasePolicyResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[UpdateBasePolicyResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        UpdateBasePolicyResponse,
        List[UpdateBasePolicyResponse],
        Dict[Any, UpdateBasePolicyResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "id": "id_",
            "affectedClientIds": "affected_client_ids",
            "affectedCountries": "affected_countries",
            "createdAt": "created_at",
            "description": "description",
            "globalPolicyName": "global_policy_name",
            "namespace": "namespace",
            "policyId": "policy_id",
            "tags": "tags",
            "typeId": "type_id",
            "updatedAt": "updated_at",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "id": True,
            "affectedClientIds": False,
            "affectedCountries": False,
            "createdAt": False,
            "description": False,
            "globalPolicyName": False,
            "namespace": False,
            "policyId": False,
            "tags": False,
            "typeId": False,
            "updatedAt": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Group Service (2.16.3)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.models_rule_response_v1 import ModelsRuleResponseV1


class ModelsUpdateGroupConfigurationResponseV1(Model):
    """Models update group configuration response V1 (models.UpdateGroupConfigurationResponseV1)

    Properties:
        allow_multiple: (allowMultiple) REQUIRED bool

        configuration_code: (configurationCode) REQUIRED str

        description: (description) REQUIRED str

        global_rules: (globalRules) REQUIRED List[ModelsRuleResponseV1]

        group_admin_role_id: (groupAdminRoleId) REQUIRED str

        group_max_member: (groupMaxMember) REQUIRED int

        group_member_role_id: (groupMemberRoleId) REQUIRED str

        name: (name) REQUIRED str
    """

    # region fields

    allow_multiple: bool  # REQUIRED
    configuration_code: str  # REQUIRED
    description: str  # REQUIRED
    global_rules: List[ModelsRuleResponseV1]  # REQUIRED
    group_admin_role_id: str  # REQUIRED
    group_max_member: int  # REQUIRED
    group_member_role_id: str  # REQUIRED
    name: str  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_allow_multiple(
        self, value: bool
    ) -> ModelsUpdateGroupConfigurationResponseV1:
        self.allow_multiple = value
        return self

    def with_configuration_code(
        self, value: str
    ) -> ModelsUpdateGroupConfigurationResponseV1:
        self.configuration_code = value
        return self

    def with_description(self, value: str) -> ModelsUpdateGroupConfigurationResponseV1:
        self.description = value
        return self

    def with_global_rules(
        self, value: List[ModelsRuleResponseV1]
    ) -> ModelsUpdateGroupConfigurationResponseV1:
        self.global_rules = value
        return self

    def with_group_admin_role_id(
        self, value: str
    ) -> ModelsUpdateGroupConfigurationResponseV1:
        self.group_admin_role_id = value
        return self

    def with_group_max_member(
        self, value: int
    ) -> ModelsUpdateGroupConfigurationResponseV1:
        self.group_max_member = value
        return self

    def with_group_member_role_id(
        self, value: str
    ) -> ModelsUpdateGroupConfigurationResponseV1:
        self.group_member_role_id = value
        return self

    def with_name(self, value: str) -> ModelsUpdateGroupConfigurationResponseV1:
        self.name = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "allow_multiple"):
            result["allowMultiple"] = bool(self.allow_multiple)
        elif include_empty:
            result["allowMultiple"] = False
        if hasattr(self, "configuration_code"):
            result["configurationCode"] = str(self.configuration_code)
        elif include_empty:
            result["configurationCode"] = ""
        if hasattr(self, "description"):
            result["description"] = str(self.description)
        elif include_empty:
            result["description"] = ""
        if hasattr(self, "global_rules"):
            result["globalRules"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.global_rules
            ]
        elif include_empty:
            result["globalRules"] = []
        if hasattr(self, "group_admin_role_id"):
            result["groupAdminRoleId"] = str(self.group_admin_role_id)
        elif include_empty:
            result["groupAdminRoleId"] = ""
        if hasattr(self, "group_max_member"):
            result["groupMaxMember"] = int(self.group_max_member)
        elif include_empty:
            result["groupMaxMember"] = 0
        if hasattr(self, "group_member_role_id"):
            result["groupMemberRoleId"] = str(self.group_member_role_id)
        elif include_empty:
            result["groupMemberRoleId"] = ""
        if hasattr(self, "name"):
            result["name"] = str(self.name)
        elif include_empty:
            result["name"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        allow_multiple: bool,
        configuration_code: str,
        description: str,
        global_rules: List[ModelsRuleResponseV1],
        group_admin_role_id: str,
        group_max_member: int,
        group_member_role_id: str,
        name: str,
        **kwargs,
    ) -> ModelsUpdateGroupConfigurationResponseV1:
        instance = cls()
        instance.allow_multiple = allow_multiple
        instance.configuration_code = configuration_code
        instance.description = description
        instance.global_rules = global_rules
        instance.group_admin_role_id = group_admin_role_id
        instance.group_max_member = group_max_member
        instance.group_member_role_id = group_member_role_id
        instance.name = name
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsUpdateGroupConfigurationResponseV1:
        instance = cls()
        if not dict_:
            return instance
        if "allowMultiple" in dict_ and dict_["allowMultiple"] is not None:
            instance.allow_multiple = bool(dict_["allowMultiple"])
        elif include_empty:
            instance.allow_multiple = False
        if "configurationCode" in dict_ and dict_["configurationCode"] is not None:
            instance.configuration_code = str(dict_["configurationCode"])
        elif include_empty:
            instance.configuration_code = ""
        if "description" in dict_ and dict_["description"] is not None:
            instance.description = str(dict_["description"])
        elif include_empty:
            instance.description = ""
        if "globalRules" in dict_ and dict_["globalRules"] is not None:
            instance.global_rules = [
                ModelsRuleResponseV1.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["globalRules"]
            ]
        elif include_empty:
            instance.global_rules = []
        if "groupAdminRoleId" in dict_ and dict_["groupAdminRoleId"] is not None:
            instance.group_admin_role_id = str(dict_["groupAdminRoleId"])
        elif include_empty:
            instance.group_admin_role_id = ""
        if "groupMaxMember" in dict_ and dict_["groupMaxMember"] is not None:
            instance.group_max_member = int(dict_["groupMaxMember"])
        elif include_empty:
            instance.group_max_member = 0
        if "groupMemberRoleId" in dict_ and dict_["groupMemberRoleId"] is not None:
            instance.group_member_role_id = str(dict_["groupMemberRoleId"])
        elif include_empty:
            instance.group_member_role_id = ""
        if "name" in dict_ and dict_["name"] is not None:
            instance.name = str(dict_["name"])
        elif include_empty:
            instance.name = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsUpdateGroupConfigurationResponseV1]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsUpdateGroupConfigurationResponseV1]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsUpdateGroupConfigurationResponseV1,
        List[ModelsUpdateGroupConfigurationResponseV1],
        Dict[Any, ModelsUpdateGroupConfigurationResponseV1],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "allowMultiple": "allow_multiple",
            "configurationCode": "configuration_code",
            "description": "description",
            "globalRules": "global_rules",
            "groupAdminRoleId": "group_admin_role_id",
            "groupMaxMember": "group_max_member",
            "groupMemberRoleId": "group_member_role_id",
            "name": "name",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "allowMultiple": True,
            "configurationCode": True,
            "description": True,
            "globalRules": True,
            "groupAdminRoleId": True,
            "groupMaxMember": True,
            "groupMemberRoleId": True,
            "name": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Iam Service."""

__version__ = "6.0.1"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .admin_ban_device_v4 import AdminBanDeviceV4
from .admin_decrypt_device_v4 import AdminDecryptDeviceV4
from .admin_generate_report_v4 import AdminGenerateReportV4
from .admin_get_banned_devices_v4 import AdminGetBannedDevicesV4
from .admin_get_device_ban_v4 import AdminGetDeviceBanV4
from .admin_get_device_bans_v4 import AdminGetDeviceBansV4
from .admin_get_device_types_v4 import AdminGetDeviceTypesV4
from .admin_get_devices_by_user_v4 import AdminGetDevicesByUserV4
from .admin_get_user_device_bans_v4 import AdminGetUserDeviceBansV4
from .admin_get_users_by_device_v4 import AdminGetUsersByDeviceV4
from .admin_unban_device_v4 import AdminUnbanDeviceV4
from .admin_update_device_ban_v4 import AdminUpdateDeviceBanV4



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Legal Service."""

__version__ = "1.29.2"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .create_localized_policy_b58242 import CreateLocalizedPolicyVersion1
from .request_presigned_url_1 import RequestPresignedURL1
from .retrieve_localized_poli_4a4859 import RetrieveLocalizedPolicyVersions1
from .retrieve_single_localiz_f21ac4 import RetrieveSingleLocalizedPolicyVersion1
from .retrieve_single_localiz_480659 import RetrieveSingleLocalizedPolicyVersion3
from .set_default_policy_1 import SetDefaultPolicy1
from .update_localized_policy_b28b13 import UpdateLocalizedPolicyVersion1



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Matchmaking Service (2.25.1)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.models_alliance_flexing_rule import ModelsAllianceFlexingRule
from ..models.models_alliance_rule_v1 import ModelsAllianceRuleV1
from ..models.models_bucket_mmr_rule import ModelsBucketMMRRule
from ..models.models_flexing_rule import ModelsFlexingRule
from ..models.models_match_option_rule import ModelsMatchOptionRule
from ..models.models_matching_rule import ModelsMatchingRule
from ..models.models_sub_game_mode import ModelsSubGameMode


class ModelsRuleSetV1(Model):
    """Models rule set V1 (models.RuleSetV1)

    Properties:
        alliance: (alliance) REQUIRED ModelsAllianceRuleV1

        alliance_flexing_rule: (alliance_flexing_rule) REQUIRED List[ModelsAllianceFlexingRule]

        flexing_rules: (flexingRules) REQUIRED List[ModelsFlexingRule]

        match_options: (match_options) REQUIRED ModelsMatchOptionRule

        matching_rules: (matchingRules) REQUIRED List[ModelsMatchingRule]

        sub_game_modes: (sub_game_modes) REQUIRED Dict[str, ModelsSubGameMode]

        bucket_mmr_rule: (bucket_mmr_rule) OPTIONAL ModelsBucketMMRRule

        use_newest_ticket_for_flexing: (use_newest_ticket_for_flexing) OPTIONAL bool
    """

    # region fields

    alliance: ModelsAllianceRuleV1  # REQUIRED
    alliance_flexing_rule: List[ModelsAllianceFlexingRule]  # REQUIRED
    flexing_rules: List[ModelsFlexingRule]  # REQUIRED
    match_options: ModelsMatchOptionRule  # REQUIRED
    matching_rules: List[ModelsMatchingRule]  # REQUIRED
    sub_game_modes: Dict[str, ModelsSubGameMode]  # REQUIRED
    bucket_mmr_rule: ModelsBucketMMRRule  # OPTIONAL
    use_newest_ticket_for_flexing: bool  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_alliance(self, value: ModelsAllianceRuleV1) -> ModelsRuleSetV1:
        self.alliance = value
        return self

    def with_alliance_flexing_rule(
        self, value: List[ModelsAllianceFlexingRule]
    ) -> ModelsRuleSetV1:
        self.alliance_flexing_rule = value
        return self

    def with_flexing_rules(self, value: List[ModelsFlexingRule]) -> ModelsRuleSetV1:
        self.flexing_rules = value
        return self

    def with_match_options(self, value: ModelsMatchOptionRule) -> ModelsRuleSetV1:
        self.match_options = value
        return self

    def with_matching_rules(self, value: List[ModelsMatchingRule]) -> ModelsRuleSetV1:
        self.matching_rules = value
        return self

    def with_sub_game_modes(
        self, value: Dict[str, ModelsSubGameMode]
    ) -> ModelsRuleSetV1:
        self.sub_game_modes = value
        return self

    def with_bucket_mmr_rule(self, value: ModelsBucketMMRRule) -> ModelsRuleSetV1:
        self.bucket_mmr_rule = value
        return self

    def with_use_newest_ticket_for_flexing(self, value: bool) -> ModelsRuleSetV1:
        self.use_newest_ticket_for_flexing = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "alliance"):
            result["alliance"] = self.alliance.to_dict(include_empty=include_empty)
        elif include_empty:
            result["alliance"] = ModelsAllianceRuleV1()
        if hasattr(self, "alliance_flexing_rule"):
            result["alliance_flexing_rule"] = [
                i0.to_dict(include_empty=include_empty)
                for i0 in self.alliance_flexing_rule
            ]
        elif include_empty:
            result["alliance_flexing_rule"] = []
        if hasattr(self, "flexing_rules"):
            result["flexingRules"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.flexing_rules
            ]
        elif include_empty:
            result["flexingRules"] = []
        if hasattr(self, "match_options"):
            result["match_options"] = self.match_options.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["match_options"] = ModelsMatchOptionRule()
        if hasattr(self, "matching_rules"):
            result["matchingRules"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.matching_rules
            ]
        elif include_empty:
            result["matchingRules"] = []
        if hasattr(self, "sub_game_modes"):
            result["sub_game_modes"] = {
                str(k0): v0.to_dict(include_empty=include_empty)
                for k0, v0 in self.sub_game_modes.items()
            }
        elif include_empty:
            result["sub_game_modes"] = {}
        if hasattr(self, "bucket_mmr_rule"):
            result["bucket_mmr_rule"] = self.bucket_mmr_rule.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["bucket_mmr_rule"] = ModelsBucketMMRRule()
        if hasattr(self, "use_newest_ticket_for_flexing"):
            result["use_newest_ticket_for_flexing"] = bool(
                self.use_newest_ticket_for_flexing
            )
        elif include_empty:
            result["use_newest_ticket_for_flexing"] = False
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        alliance: ModelsAllianceRuleV1,
        alliance_flexing_rule: List[ModelsAllianceFlexingRule],
        flexing_rules: List[ModelsFlexingRule],
        match_options: ModelsMatchOptionRule,
        matching_rules: List[ModelsMatchingRule],
        sub_game_modes: Dict[str, ModelsSubGameMode],
        bucket_mmr_rule: Optional[ModelsBucketMMRRule] = None,
        use_newest_ticket_for_flexing: Optional[bool] = None,
        **kwargs,
    ) -> ModelsRuleSetV1:
        instance = cls()
        instance.alliance = alliance
        instance.alliance_flexing_rule = alliance_flexing_rule
        instance.flexing_rules = flexing_rules
        instance.match_options = match_options
        instance.matching_rules = matching_rules
        instance.sub_game_modes = sub_game_modes
        if bucket_mmr_rule is not None:
            instance.bucket_mmr_rule = bucket_mmr_rule
        if use_newest_ticket_for_flexing is not None:
            instance.use_newest_ticket_for_flexing = use_newest_ticket_for_flexing
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsRuleSetV1:
        instance = cls()
        if not dict_:
            return instance
        if "alliance" in dict_ and dict_["alliance"] is not None:
            instance.alliance = ModelsAllianceRuleV1.create_from_dict(
                dict_["alliance"], include_empty=include_empty
            )
        elif include_empty:
            instance.alliance = ModelsAllianceRuleV1()
        if (
            "alliance_flexing_rule" in dict_
            and dict_["alliance_flexing_rule"] is not None
        ):
            instance.alliance_flexing_rule = [
                ModelsAllianceFlexingRule.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["alliance_flexing_rule"]
            ]
        elif include_empty:
            instance.alliance_flexing_rule = []
        if "flexingRules" in dict_ and dict_["flexingRules"] is not None:
            instance.flexing_rules = [
                ModelsFlexingRule.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["flexingRules"]
            ]
        elif include_empty:
            instance.flexing_rules = []
        if "match_options" in dict_ and dict_["match_options"] is not None:
            instance.match_options = ModelsMatchOptionRule.create_from_dict(
                dict_["match_options"], include_empty=include_empty
            )
        elif include_empty:
            instance.match_options = ModelsMatchOptionRule()
        if "matchingRules" in dict_ and dict_["matchingRules"] is not None:
            instance.matching_rules = [
                ModelsMatchingRule.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["matchingRules"]
            ]
        elif include_empty:
            instance.matching_rules = []
        if "sub_game_modes" in dict_ and dict_["sub_game_modes"] is not None:
            instance.sub_game_modes = {
                str(k0): ModelsSubGameMode.create_from_dict(
                    v0, include_empty=include_empty
                )
                for k0, v0 in dict_["sub_game_modes"].items()
            }
        elif include_empty:
            instance.sub_game_modes = {}
        if "bucket_mmr_rule" in dict_ and dict_["bucket_mmr_rule"] is not None:
            instance.bucket_mmr_rule = ModelsBucketMMRRule.create_from_dict(
                dict_["bucket_mmr_rule"], include_empty=include_empty
            )
        elif include_empty:
            instance.bucket_mmr_rule = ModelsBucketMMRRule()
        if (
            "use_newest_ticket_for_flexing" in dict_
            and dict_["use_newest_ticket_for_flexing"] is not None
        ):
            instance.use_newest_ticket_for_flexing = bool(
                dict_["use_newest_ticket_for_flexing"]
            )
        elif include_empty:
            instance.use_newest_ticket_for_flexing = False
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsRuleSetV1]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsRuleSetV1]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[ModelsRuleSetV1, List[ModelsRuleSetV1], Dict[Any, ModelsRuleSetV1]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "alliance": "alliance",
            "alliance_flexing_rule": "alliance_flexing_rule",
            "flexingRules": "flexing_rules",
            "match_options": "match_options",
            "matchingRules": "matching_rules",
            "sub_game_modes": "sub_game_modes",
            "bucket_mmr_rule": "bucket_mmr_rule",
            "use_newest_ticket_for_flexing": "use_newest_ticket_for_flexing",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "alliance": True,
            "alliance_flexing_rule": True,
            "flexingRules": True,
            "match_options": True,
            "matchingRules": True,
            "sub_game_modes": True,
            "bucket_mmr_rule": False,
            "use_newest_ticket_for_flexing": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Matchmaking Service (2.25.1)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.models_mock_match import ModelsMockMatch


class ModelsGetMockMatchesResponse(Model):
    """Models get mock matches response (models.GetMockMatchesResponse)

    Properties:
        count: (count) REQUIRED int

        largest_mmr_distance: (largest_mmr_distance) REQUIRED int

        longest_time_to_match_seconds: (longest_time_to_match_seconds) REQUIRED int

        matches: (matches) REQUIRED List[ModelsMockMatch]
    """

    # region fields

    count: int  # REQUIRED
    largest_mmr_distance: int  # REQUIRED
    longest_time_to_match_seconds: int  # REQUIRED
    matches: List[ModelsMockMatch]  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_count(self, value: int) -> ModelsGetMockMatchesResponse:
        self.count = value
        return self

    def with_largest_mmr_distance(self, value: int) -> ModelsGetMockMatchesResponse:
        self.largest_mmr_distance = value
        return self

    def with_longest_time_to_match_seconds(
        self, value: int
    ) -> ModelsGetMockMatchesResponse:
        self.longest_time_to_match_seconds = value
        return self

    def with_matches(
        self, value: List[ModelsMockMatch]
    ) -> ModelsGetMockMatchesResponse:
        self.matches = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "count"):
            result["count"] = int(self.count)
        elif include_empty:
            result["count"] = 0
        if hasattr(self, "largest_mmr_distance"):
            result["largest_mmr_distance"] = int(self.largest_mmr_distance)
        elif include_empty:
            result["largest_mmr_distance"] = 0
        if hasattr(self, "longest_time_to_match_seconds"):
            result["longest_time_to_match_seconds"] = int(
                self.longest_time_to_match_seconds
            )
        elif include_empty:
            result["longest_time_to_match_seconds"] = 0
        if hasattr(self, "matches"):
            result["matches"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.matches
            ]
        elif include_empty:
            result["matches"] = []
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        count: int,
        largest_mmr_distance: int,
        longest_time_to_match_seconds: int,
        matches: List[ModelsMockMatch],
        **kwargs,
    ) -> ModelsGetMockMatchesResponse:
        instance = cls()
        instance.count = count
        instance.largest_mmr_distance = largest_mmr_distance
        instance.longest_time_to_match_seconds = longest_time_to_match_seconds
        instance.matches = matches
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsGetMockMatchesResponse:
        instance = cls()
        if not dict_:
            return instance
        if "count" in dict_ and dict_["count"] is not None:
            instance.count = int(dict_["count"])
        elif include_empty:
            instance.count = 0
        if (
            "largest_mmr_distance" in dict_
            and dict_["largest_mmr_distance"] is not None
        ):
            instance.largest_mmr_distance = int(dict_["largest_mmr_distance"])
        elif include_empty:
            instance.largest_mmr_distance = 0
        if (
            "longest_time_to_match_seconds" in dict_
            and dict_["longest_time_to_match_seconds"] is not None
        ):
            instance.longest_time_to_match_seconds = int(
                dict_["longest_time_to_match_seconds"]
            )
        elif include_empty:
            instance.longest_time_to_match_seconds = 0
        if "matches" in dict_ and dict_["matches"] is not None:
            instance.matches = [
                ModelsMockMatch.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["matches"]
            ]
        elif include_empty:
            instance.matches = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsGetMockMatchesResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsGetMockMatchesResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsGetMockMatchesResponse,
        List[ModelsGetMockMatchesResponse],
        Dict[Any, ModelsGetMockMatchesResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "count": "count",
            "largest_mmr_distance": "largest_mmr_distance",
            "longest_time_to_match_seconds": "longest_time_to_match_seconds",
            "matches": "matches",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "count": True,
            "largest_mmr_distance": True,
            "longest_time_to_match_seconds": True,
            "matches": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Iam Service (6.0.1)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.accountcommon_avatar_config import AccountcommonAvatarConfig
from ..models.accountcommon_input_validation_description import (
    AccountcommonInputValidationDescription,
)


class ModelValidationDetailPublic(Model):
    """Model validation detail public (model.ValidationDetailPublic)

    Properties:
        allow_all_special_characters: (allowAllSpecialCharacters) REQUIRED bool

        allow_digit: (allowDigit) REQUIRED bool

        allow_letter: (allowLetter) REQUIRED bool

        allow_space: (allowSpace) REQUIRED bool

        allow_unicode: (allowUnicode) REQUIRED bool

        blocked_word: (blockedWord) REQUIRED List[str]

        description: (description) REQUIRED AccountcommonInputValidationDescription

        is_custom_regex: (isCustomRegex) REQUIRED bool

        letter_case: (letterCase) REQUIRED str

        max_length: (maxLength) REQUIRED int

        max_repeating_alpha_num: (maxRepeatingAlphaNum) REQUIRED int

        max_repeating_special_character: (maxRepeatingSpecialCharacter) REQUIRED int

        min_char_type: (minCharType) REQUIRED int

        min_length: (minLength) REQUIRED int

        regex: (regex) REQUIRED str

        special_character_location: (specialCharacterLocation) REQUIRED str

        special_characters: (specialCharacters) REQUIRED List[str]

        avatar_config: (avatarConfig) OPTIONAL AccountcommonAvatarConfig
    """

    # region fields

    allow_all_special_characters: bool  # REQUIRED
    allow_digit: bool  # REQUIRED
    allow_letter: bool  # REQUIRED
    allow_space: bool  # REQUIRED
    allow_unicode: bool  # REQUIRED
    blocked_word: List[str]  # REQUIRED
    description: AccountcommonInputValidationDescription  # REQUIRED
    is_custom_regex: bool  # REQUIRED
    letter_case: str  # REQUIRED
    max_length: int  # REQUIRED
    max_repeating_alpha_num: int  # REQUIRED
    max_repeating_special_character: int  # REQUIRED
    min_char_type: int  # REQUIRED
    min_length: int  # REQUIRED
    regex: str  # REQUIRED
    special_character_location: str  # REQUIRED
    special_characters: List[str]  # REQUIRED
    avatar_config: AccountcommonAvatarConfig  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_allow_all_special_characters(
        self, value: bool
    ) -> ModelValidationDetailPublic:
        self.allow_all_special_characters = value
        return self

    def with_allow_digit(self, value: bool) -> ModelValidationDetailPublic:
        self.allow_digit = value
        return self

    def with_allow_letter(self, value: bool) -> ModelValidationDetailPublic:
        self.allow_letter = value
        return self

    def with_allow_space(self, value: bool) -> ModelValidationDetailPublic:
        self.allow_space = value
        return self

    def with_allow_unicode(self, value: bool) -> ModelValidationDetailPublic:
        self.allow_unicode = value
        return self

    def with_blocked_word(self, value: List[str]) -> ModelValidationDetailPublic:
        self.blocked_word = value
        return self

    def with_description(
        self, value: AccountcommonInputValidationDescription
    ) -> ModelValidationDetailPublic:
        self.description = value
        return self

    def with_is_custom_regex(self, value: bool) -> ModelValidationDetailPublic:
        self.is_custom_regex = value
        return self

    def with_letter_case(self, value: str) -> ModelValidationDetailPublic:
        self.letter_case = value
        return self

    def with_max_length(self, value: int) -> ModelValidationDetailPublic:
        self.max_length = value
        return self

    def with_max_repeating_alpha_num(self, value: int) -> ModelValidationDetailPublic:
        self.max_repeating_alpha_num = value
        return self

    def with_max_repeating_special_character(
        self, value: int
    ) -> ModelValidationDetailPublic:
        self.max_repeating_special_character = value
        return self

    def with_min_char_type(self, value: int) -> ModelValidationDetailPublic:
        self.min_char_type = value
        return self

    def with_min_length(self, value: int) -> ModelValidationDetailPublic:
        self.min_length = value
        return self

    def with_regex(self, value: str) -> ModelValidationDetailPublic:
        self.regex = value
        return self

    def with_special_character_location(
        self, value: str
    ) -> ModelValidationDetailPublic:
        self.special_character_location = value
        return self

    def with_special_characters(self, value: List[str]) -> ModelValidationDetailPublic:
        self.special_characters = value
        return self

    def with_avatar_config(
        self, value: AccountcommonAvatarConfig
    ) -> ModelValidationDetailPublic:
        self.avatar_config = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "allow_all_special_characters"):
            result["allowAllSpecialCharacters"] = bool(
                self.allow_all_special_characters
            )
        elif include_empty:
            result["allowAllSpecialCharacters"] = False
        if hasattr(self, "allow_digit"):
            result["allowDigit"] = bool(self.allow_digit)
        elif include_empty:
            result["allowDigit"] = False
        if hasattr(self, "allow_letter"):
            result["allowLetter"] = bool(self.allow_letter)
        elif include_empty:
            result["allowLetter"] = False
        if hasattr(self, "allow_space"):
            result["allowSpace"] = bool(self.allow_space)
        elif include_empty:
            result["allowSpace"] = False
        if hasattr(self, "allow_unicode"):
            result["allowUnicode"] = bool(self.allow_unicode)
        elif include_empty:
            result["allowUnicode"] = False
        if hasattr(self, "blocked_word"):
            result["blockedWord"] = [str(i0) for i0 in self.blocked_word]
        elif include_empty:
            result["blockedWord"] = []
        if hasattr(self, "description"):
            result["description"] = self.description.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["description"] = AccountcommonInputValidationDescription()
        if hasattr(self, "is_custom_regex"):
            result["isCustomRegex"] = bool(self.is_custom_regex)
        elif include_empty:
            result["isCustomRegex"] = False
        if hasattr(self, "letter_case"):
            result["letterCase"] = str(self.letter_case)
        elif include_empty:
            result["letterCase"] = ""
        if hasattr(self, "max_length"):
            result["maxLength"] = int(self.max_length)
        elif include_empty:
            result["maxLength"] = 0
        if hasattr(self, "max_repeating_alpha_num"):
            result["maxRepeatingAlphaNum"] = int(self.max_repeating_alpha_num)
        elif include_empty:
            result["maxRepeatingAlphaNum"] = 0
        if hasattr(self, "max_repeating_special_character"):
            result["maxRepeatingSpecialCharacter"] = int(
                self.max_repeating_special_character
            )
        elif include_empty:
            result["maxRepeatingSpecialCharacter"] = 0
        if hasattr(self, "min_char_type"):
            result["minCharType"] = int(self.min_char_type)
        elif include_empty:
            result["minCharType"] = 0
        if hasattr(self, "min_length"):
            result["minLength"] = int(self.min_length)
        elif include_empty:
            result["minLength"] = 0
        if hasattr(self, "regex"):
            result["regex"] = str(self.regex)
        elif include_empty:
            result["regex"] = ""
        if hasattr(self, "special_character_location"):
            result["specialCharacterLocation"] = str(self.special_character_location)
        elif include_empty:
            result["specialCharacterLocation"] = ""
        if hasattr(self, "special_characters"):
            result["specialCharacters"] = [str(i0) for i0 in self.special_characters]
        elif include_empty:
            result["specialCharacters"] = []
        if hasattr(self, "avatar_config"):
            result["avatarConfig"] = self.avatar_config.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["avatarConfig"] = AccountcommonAvatarConfig()
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        allow_all_special_characters: bool,
        allow_digit: bool,
        allow_letter: bool,
        allow_space: bool,
        allow_unicode: bool,
        blocked_word: List[str],
        description: AccountcommonInputValidationDescription,
        is_custom_regex: bool,
        letter_case: str,
        max_length: int,
        max_repeating_alpha_num: int,
        max_repeating_special_character: int,
        min_char_type: int,
        min_length: int,
        regex: str,
        special_character_location: str,
        special_characters: List[str],
        avatar_config: Optional[AccountcommonAvatarConfig] = None,
        **kwargs,
    ) -> ModelValidationDetailPublic:
        instance = cls()
        instance.allow_all_special_characters = allow_all_special_characters
        instance.allow_digit = allow_digit
        instance.allow_letter = allow_letter
        instance.allow_space = allow_space
        instance.allow_unicode = allow_unicode
        instance.blocked_word = blocked_word
        instance.description = description
        instance.is_custom_regex = is_custom_regex
        instance.letter_case = letter_case
        instance.max_length = max_length
        instance.max_repeating_alpha_num = max_repeating_alpha_num
        instance.max_repeating_special_character = max_repeating_special_character
        instance.min_char_type = min_char_type
        instance.min_length = min_length
        instance.regex = regex
        instance.special_character_location = special_character_location
        instance.special_characters = special_characters
        if avatar_config is not None:
            instance.avatar_config = avatar_config
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelValidationDetailPublic:
        instance = cls()
        if not dict_:
            return instance
        if (
            "allowAllSpecialCharacters" in dict_
            and dict_["allowAllSpecialCharacters"] is not None
        ):
            instance.allow_all_special_characters = bool(
                dict_["allowAllSpecialCharacters"]
            )
        elif include_empty:
            instance.allow_all_special_characters = False
        if "allowDigit" in dict_ and dict_["allowDigit"] is not None:
            instance.allow_digit = bool(dict_["allowDigit"])
        elif include_empty:
            instance.allow_digit = False
        if "allowLetter" in dict_ and dict_["allowLetter"] is not None:
            instance.allow_letter = bool(dict_["allowLetter"])
        elif include_empty:
            instance.allow_letter = False
        if "allowSpace" in dict_ and dict_["allowSpace"] is not None:
            instance.allow_space = bool(dict_["allowSpace"])
        elif include_empty:
            instance.allow_space = False
        if "allowUnicode" in dict_ and dict_["allowUnicode"] is not None:
            instance.allow_unicode = bool(dict_["allowUnicode"])
        elif include_empty:
            instance.allow_unicode = False
        if "blockedWord" in dict_ and dict_["blockedWord"] is not None:
            instance.blocked_word = [str(i0) for i0 in dict_["blockedWord"]]
        elif include_empty:
            instance.blocked_word = []
        if "description" in dict_ and dict_["description"] is not None:
            instance.description = (
                AccountcommonInputValidationDescription.create_from_dict(
                    dict_["description"], include_empty=include_empty
                )
            )
        elif include_empty:
            instance.description = AccountcommonInputValidationDescription()
        if "isCustomRegex" in dict_ and dict_["isCustomRegex"] is not None:
            instance.is_custom_regex = bool(dict_["isCustomRegex"])
        elif include_empty:
            instance.is_custom_regex = False
        if "letterCase" in dict_ and dict_["letterCase"] is not None:
            instance.letter_case = str(dict_["letterCase"])
        elif include_empty:
            instance.letter_case = ""
        if "maxLength" in dict_ and dict_["maxLength"] is not None:
            instance.max_length = int(dict_["maxLength"])
        elif include_empty:
            instance.max_length = 0
        if (
            "maxRepeatingAlphaNum" in dict_
            and dict_["maxRepeatingAlphaNum"] is not None
        ):
            instance.max_repeating_alpha_num = int(dict_["maxRepeatingAlphaNum"])
        elif include_empty:
            instance.max_repeating_alpha_num = 0
        if (
            "maxRepeatingSpecialCharacter" in dict_
            and dict_["maxRepeatingSpecialCharacter"] is not None
        ):
            instance.max_repeating_special_character = int(
                dict_["maxRepeatingSpecialCharacter"]
            )
        elif include_empty:
            instance.max_repeating_special_character = 0
        if "minCharType" in dict_ and dict_["minCharType"] is not None:
            instance.min_char_type = int(dict_["minCharType"])
        elif include_empty:
            instance.min_char_type = 0
        if "minLength" in dict_ and dict_["minLength"] is not None:
            instance.min_length = int(dict_["minLength"])
        elif include_empty:
            instance.min_length = 0
        if "regex" in dict_ and dict_["regex"] is not None:
            instance.regex = str(dict_["regex"])
        elif include_empty:
            instance.regex = ""
        if (
            "specialCharacterLocation" in dict_
            and dict_["specialCharacterLocation"] is not None
        ):
            instance.special_character_location = str(dict_["specialCharacterLocation"])
        elif include_empty:
            instance.special_character_location = ""
        if "specialCharacters" in dict_ and dict_["specialCharacters"] is not None:
            instance.special_characters = [str(i0) for i0 in dict_["specialCharacters"]]
        elif include_empty:
            instance.special_characters = []
        if "avatarConfig" in dict_ and dict_["avatarConfig"] is not None:
            instance.avatar_config = AccountcommonAvatarConfig.create_from_dict(
                dict_["avatarConfig"], include_empty=include_empty
            )
        elif include_empty:
            instance.avatar_config = AccountcommonAvatarConfig()
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelValidationDetailPublic]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelValidationDetailPublic]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelValidationDetailPublic,
        List[ModelValidationDetailPublic],
        Dict[Any, ModelValidationDetailPublic],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "allowAllSpecialCharacters": "allow_all_special_characters",
            "allowDigit": "allow_digit",
            "allowLetter": "allow_letter",
            "allowSpace": "allow_space",
            "allowUnicode": "allow_unicode",
            "blockedWord": "blocked_word",
            "description": "description",
            "isCustomRegex": "is_custom_regex",
            "letterCase": "letter_case",
            "maxLength": "max_length",
            "maxRepeatingAlphaNum": "max_repeating_alpha_num",
            "maxRepeatingSpecialCharacter": "max_repeating_special_character",
            "minCharType": "min_char_type",
            "minLength": "min_length",
            "regex": "regex",
            "specialCharacterLocation": "special_character_location",
            "specialCharacters": "special_characters",
            "avatarConfig": "avatar_config",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "allowAllSpecialCharacters": True,
            "allowDigit": True,
            "allowLetter": True,
            "allowSpace": True,
            "allowUnicode": True,
            "blockedWord": True,
            "description": True,
            "isCustomRegex": True,
            "letterCase": True,
            "maxLength": True,
            "maxRepeatingAlphaNum": True,
            "maxRepeatingSpecialCharacter": True,
            "minCharType": True,
            "minLength": True,
            "regex": True,
            "specialCharacterLocation": True,
            "specialCharacters": True,
            "avatarConfig": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ModelBulkAddFriendsRequest
from ..models import ModelGetFriendsResponse
from ..models import ModelGetUserFriendsResponse
from ..models import ModelGetUserIncomingFriendsResponse
from ..models import ModelGetUserOutgoingFriendsResponse
from ..models import ModelListBulkUserPlatformsResponse
from ..models import ModelLoadIncomingFriendsWithTimeResponse
from ..models import ModelLoadOutgoingFriendsWithTimeResponse
from ..models import ModelUserAcceptFriendRequest
from ..models import ModelUserCancelFriendRequest
from ..models import ModelUserGetFriendshipStatusResponse
from ..models import ModelUserRejectFriendRequest
from ..models import ModelUserRequestFriendRequest
from ..models import ModelUserUnfriendRequest
from ..models import RestapiErrorResponseBody
from ..models import RestapiErrorResponseV1

from ..operations.friends import AddFriendsWithoutConfirmation
from ..operations.friends import GetIncomingFriendRequests
from ..operations.friends import GetListOfFriends
from ..operations.friends import GetOutgoingFriendRequests
from ..operations.friends import GetUserFriendsUpdated
from ..operations.friends import GetUserFriendsWithPlatform
from ..operations.friends import GetUserIncomingFriends
from ..operations.friends import GetUserIncomingFriendsWithTime
from ..operations.friends import GetUserOutgoingFriends
from ..operations.friends import GetUserOutgoingFriendsWithTime
from ..operations.friends import UserAcceptFriendRequest
from ..operations.friends import UserCancelFriendRequest
from ..operations.friends import UserGetFriendshipStatus
from ..operations.friends import UserRejectFriendRequest
from ..operations.friends import UserRequestFriend
from ..operations.friends import UserUnfriendRequest


@same_doc_as(AddFriendsWithoutConfirmation)
def add_friends_without_confirmation(
    body: ModelBulkAddFriendsRequest,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Add friends without confirmation (addFriendsWithoutConfirmation)

    Required permission : `NAMESPACE:{namespace}:USER:{userId}:FRIENDS [CREATE]` with scope `social`

    friends request in a namespace.

    Required Permission(s):
        - NAMESPACE:{namespace}:USER:{userId}:FRIENDS [CREATE]

    Required Scope(s):
        - social

    Properties:
        url: /friends/namespaces/{namespace}/users/{userId}/add/bulk

        method: POST

        tags: ["friends", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelBulkAddFriendsRequest in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        204: No Content -

        400: Bad Request - RestapiErrorResponseV1 (Bad Request)

        401: Unauthorized - RestapiErrorResponseV1 (Unauthorized)

        403: Forbidden - RestapiErrorResponseV1 (Forbidden)

        500: Internal Server Error - RestapiErrorResponseV1 (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AddFriendsWithoutConfirmation.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AddFriendsWithoutConfirmation)
async def add_friends_without_confirmation_async(
    body: ModelBulkAddFriendsRequest,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Add friends without confirmation (addFriendsWithoutConfirmation)

    Required permission : `NAMESPACE:{namespace}:USER:{userId}:FRIENDS [CREATE]` with scope `social`

    friends request in a namespace.

    Required Permission(s):
        - NAMESPACE:{namespace}:USER:{userId}:FRIENDS [CREATE]

    Required Scope(s):
        - social

    Properties:
        url: /friends/namespaces/{namespace}/users/{userId}/add/bulk

        method: POST

        tags: ["friends", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelBulkAddFriendsRequest in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        204: No Content -

        400: Bad Request - RestapiErrorResponseV1 (Bad Request)

        401: Unauthorized - RestapiErrorResponseV1 (Unauthorized)

        403: Forbidden - RestapiErrorResponseV1 (Forbidden)

        500: Internal Server Error - RestapiErrorResponseV1 (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AddFriendsWithoutConfirmation.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetIncomingFriendRequests)
def get_incoming_friend_requests(
    user_id: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """get incoming friend requests (get incoming friend requests)

    Required permission : `NAMESPACE:{namespace}:USER:{userId}:FRIENDS [READ]` with scope `social`

    get list of incoming friend requests.

    Required Permission(s):
        - NAMESPACE:{namespace}:USER:{userId}:FRIENDS [READ]

    Required Scope(s):
        - social

    Properties:
        url: /lobby/v1/admin/friend/namespaces/{namespace}/users/{userId}/incoming

        method: GET

        tags: ["friends"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - ModelLoadIncomingFriendsWithTimeResponse

        400: Bad Request - RestapiErrorResponseBody (Bad Request)

        401: Unauthorized - RestapiErrorResponseBody (Unauthorized)

        403: Forbidden - RestapiErrorResponseBody (Forbidden)

        500: Internal Server Error - RestapiErrorResponseBody (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetIncomingFriendRequests.create(
        user_id=user_id,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetIncomingFriendRequests)
async def get_incoming_friend_requests_async(
    user_id: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """get incoming friend requests (get incoming friend requests)

    Required permission : `NAMESPACE:{namespace}:USER:{userId}:FRIENDS [READ]` with scope `social`

    get list of incoming friend requests.

    Required Permission(s):
        - NAMESPACE:{namespace}:USER:{userId}:FRIENDS [READ]

    Required Scope(s):
        - social

    Properties:
        url: /lobby/v1/admin/friend/namespaces/{namespace}/users/{userId}/incoming

        method: GET

        tags: ["friends"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - ModelLoadIncomingFriendsWithTimeResponse

        400: Bad Request - RestapiErrorResponseBody (Bad Request)

        401: Unauthorized - RestapiErrorResponseBody (Unauthorized)

        403: Forbidden - RestapiErrorResponseBody (Forbidden)

        500: Internal Server Error - RestapiErrorResponseBody (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetIncomingFriendRequests.create(
        user_id=user_id,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetListOfFriends)
def get_list_of_friends(
    user_id: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """get list of friends (get list of friends)

    Required permission : `NAMESPACE:{namespace}:USER:{userId}:FRIENDS [READ]` with scope `social`

    get list of friends in a namespace.

    Required Permission(s):
        - NAMESPACE:{namespace}:USER:{userId}:FRIENDS [READ]

    Required Scope(s):
        - social

    Properties:
        url: /lobby/v1/admin/friend/namespaces/{namespace}/users/{userId}

        method: GET

        tags: ["friends"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - ModelGetFriendsResponse

        400: Bad Request - RestapiErrorResponseBody (Bad Request)

        401: Unauthorized - RestapiErrorResponseBody (Unauthorized)

        403: Forbidden - RestapiErrorResponseBody (Forbidden)

        500: Internal Server Error - RestapiErrorResponseBody (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetListOfFriends.create(
        user_id=user_id,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetListOfFriends)
async def get_list_of_friends_async(
    user_id: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """get list of friends (get list of friends)

    Required permission : `NAMESPACE:{namespace}:USER:{userId}:FRIENDS [READ]` with scope `social`

    get list of friends in a namespace.

    Required Permission(s):
        - NAMESPACE:{namespace}:USER:{userId}:FRIENDS [READ]

    Required Scope(s):
        - social

    Properties:
        url: /lobby/v1/admin/friend/namespaces/{namespace}/users/{userId}

        method: GET

        tags: ["friends"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - ModelGetFriendsResponse

        400: Bad Request - RestapiErrorResponseBody (Bad Request)

        401: Unauthorized - RestapiErrorResponseBody (Unauthorized)

        403: Forbidden - RestapiErrorResponseBody (Forbidden)

        500: Internal Server Error - RestapiErrorResponseBody (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetListOfFriends.create(
        user_id=user_id,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetOutgoingFriendRequests)
def get_outgoing_friend_requests(
    user_id: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """get list of outgoing friend requests (get outgoing friend requests)

    Required permission : `NAMESPACE:{namespace}:USER:{userId}:FRIENDS [READ]` with scope `social`

    get list of outgoing friend requests in a namespace.

    Required Permission(s):
        - NAMESPACE:{namespace}:USER:{userId}:FRIENDS [READ]

    Required Scope(s):
        - social

    Properties:
        url: /lobby/v1/admin/friend/namespaces/{namespace}/users/{userId}/outgoing

        method: GET

        tags: ["friends"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - ModelLoadOutgoingFriendsWithTimeResponse

        400: Bad Request - RestapiErrorResponseBody (Bad Request)

        401: Unauthorized - RestapiErrorResponseBody (Unauthorized)

        403: Forbidden - RestapiErrorResponseBody (Forbidden)

        500: Internal Server Error - RestapiErrorResponseBody (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetOutgoingFriendRequests.create(
        user_id=user_id,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetOutgoingFriendRequests)
async def get_outgoing_friend_requests_async(
    user_id: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """get list of outgoing friend requests (get outgoing friend requests)

    Required permission : `NAMESPACE:{namespace}:USER:{userId}:FRIENDS [READ]` with scope `social`

    get list of outgoing friend requests in a namespace.

    Required Permission(s):
        - NAMESPACE:{namespace}:USER:{userId}:FRIENDS [READ]

    Required Scope(s):
        - social

    Properties:
        url: /lobby/v1/admin/friend/namespaces/{namespace}/users/{userId}/outgoing

        method: GET

        tags: ["friends"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - ModelLoadOutgoingFriendsWithTimeResponse

        400: Bad Request - RestapiErrorResponseBody (Bad Request)

        401: Unauthorized - RestapiErrorResponseBody (Unauthorized)

        403: Forbidden - RestapiErrorResponseBody (Forbidden)

        500: Internal Server Error - RestapiErrorResponseBody (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetOutgoingFriendRequests.create(
        user_id=user_id,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetUserFriendsUpdated)
def get_user_friends_updated(
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """get list of friends (getUserFriendsUpdated)

    Properties:
        url: /friends/namespaces/{namespace}/me

        method: GET

        tags: ["friends", "public"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - List[ModelGetUserFriendsResponse] (OK)

        400: Bad Request - RestapiErrorResponseV1 (Bad Request)

        401: Unauthorized - RestapiErrorResponseV1 (Unauthorized)

        403: Forbidden - RestapiErrorResponseV1 (Forbidden)

        404: Not Found - RestapiErrorResponseV1 (Not Found)

        500: Internal Server Error - RestapiErrorResponseV1 (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetUserFriendsUpdated.create(
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetUserFriendsUpdated)
async def get_user_friends_updated_async(
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """get list of friends (getUserFriendsUpdated)

    Properties:
        url: /friends/namespaces/{namespace}/me

        method: GET

        tags: ["friends", "public"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - List[ModelGetUserFriendsResponse] (OK)

        400: Bad Request - RestapiErrorResponseV1 (Bad Request)

        401: Unauthorized - RestapiErrorResponseV1 (Unauthorized)

        403: Forbidden - RestapiErrorResponseV1 (Forbidden)

        404: Not Found - RestapiErrorResponseV1 (Not Found)

        500: Internal Server Error - RestapiErrorResponseV1 (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetUserFriendsUpdated.create(
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetUserFriendsWithPlatform)
def get_user_friends_with_platform(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """get list of friends with platform data (getUserFriendsWithPlatform)

    Properties:
        url: /friends/namespaces/{namespace}/me/platforms

        method: GET

        tags: ["friends", "public"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelListBulkUserPlatformsResponse (OK)

        400: Bad Request - RestapiErrorResponseV1 (Bad Request)

        401: Unauthorized - RestapiErrorResponseV1 (Unauthorized)

        403: Forbidden - RestapiErrorResponseV1 (Forbidden)

        404: Not Found - RestapiErrorResponseV1 (Not Found)

        500: Internal Server Error - RestapiErrorResponseV1 (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetUserFriendsWithPlatform.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetUserFriendsWithPlatform)
async def get_user_friends_with_platform_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """get list of friends with platform data (getUserFriendsWithPlatform)

    Properties:
        url: /friends/namespaces/{namespace}/me/platforms

        method: GET

        tags: ["friends", "public"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelListBulkUserPlatformsResponse (OK)

        400: Bad Request - RestapiErrorResponseV1 (Bad Request)

        401: Unauthorized - RestapiErrorResponseV1 (Unauthorized)

        403: Forbidden - RestapiErrorResponseV1 (Forbidden)

        404: Not Found - RestapiErrorResponseV1 (Not Found)

        500: Internal Server Error - RestapiErrorResponseV1 (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetUserFriendsWithPlatform.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetUserIncomingFriends)
def get_user_incoming_friends(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """get list of incoming friends (getUserIncomingFriends)

    Properties:
        url: /friends/namespaces/{namespace}/me/incoming

        method: GET

        tags: ["friends", "public"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - List[ModelGetUserIncomingFriendsResponse] (OK)

        400: Bad Request - RestapiErrorResponseV1 (Bad Request)

        401: Unauthorized - RestapiErrorResponseV1 (Unauthorized)

        403: Forbidden - RestapiErrorResponseV1 (Forbidden)

        404: Not Found - RestapiErrorResponseV1 (Not Found)

        500: Internal Server Error - RestapiErrorResponseV1 (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetUserIncomingFriends.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetUserIncomingFriends)
async def get_user_incoming_friends_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """get list of incoming friends (getUserIncomingFriends)

    Properties:
        url: /friends/namespaces/{namespace}/me/incoming

        method: GET

        tags: ["friends", "public"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - List[ModelGetUserIncomingFriendsResponse] (OK)

        400: Bad Request - RestapiErrorResponseV1 (Bad Request)

        401: Unauthorized - RestapiErrorResponseV1 (Unauthorized)

        403: Forbidden - RestapiErrorResponseV1 (Forbidden)

        404: Not Found - RestapiErrorResponseV1 (Not Found)

        500: Internal Server Error - RestapiErrorResponseV1 (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetUserIncomingFriends.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetUserIncomingFriendsWithTime)
def get_user_incoming_friends_with_time(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """get list of incoming friends with requested time info (getUserIncomingFriendsWithTime)

    Properties:
        url: /friends/namespaces/{namespace}/me/incoming-time

        method: GET

        tags: ["friends", "public"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - List[ModelLoadIncomingFriendsWithTimeResponse] (OK)

        400: Bad Request - RestapiErrorResponseV1 (Bad Request)

        401: Unauthorized - RestapiErrorResponseV1 (Unauthorized)

        403: Forbidden - RestapiErrorResponseV1 (Forbidden)

        404: Not Found - RestapiErrorResponseV1 (Not Found)

        500: Internal Server Error - RestapiErrorResponseV1 (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetUserIncomingFriendsWithTime.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetUserIncomingFriendsWithTime)
async def get_user_incoming_friends_with_time_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """get list of incoming friends with requested time info (getUserIncomingFriendsWithTime)

    Properties:
        url: /friends/namespaces/{namespace}/me/incoming-time

        method: GET

        tags: ["friends", "public"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - List[ModelLoadIncomingFriendsWithTimeResponse] (OK)

        400: Bad Request - RestapiErrorResponseV1 (Bad Request)

        401: Unauthorized - RestapiErrorResponseV1 (Unauthorized)

        403: Forbidden - RestapiErrorResponseV1 (Forbidden)

        404: Not Found - RestapiErrorResponseV1 (Not Found)

        500: Internal Server Error - RestapiErrorResponseV1 (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetUserIncomingFriendsWithTime.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetUserOutgoingFriends)
def get_user_outgoing_friends(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """get list of outgoing friends (getUserOutgoingFriends)

    Properties:
        url: /friends/namespaces/{namespace}/me/outgoing

        method: GET

        tags: ["friends", "public"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - List[ModelGetUserOutgoingFriendsResponse] (OK)

        400: Bad Request - RestapiErrorResponseV1 (Bad Request)

        401: Unauthorized - RestapiErrorResponseV1 (Unauthorized)

        403: Forbidden - RestapiErrorResponseV1 (Forbidden)

        404: Not Found - RestapiErrorResponseV1 (Not Found)

        500: Internal Server Error - RestapiErrorResponseV1 (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetUserOutgoingFriends.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetUserOutgoingFriends)
async def get_user_outgoing_friends_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """get list of outgoing friends (getUserOutgoingFriends)

    Properties:
        url: /friends/namespaces/{namespace}/me/outgoing

        method: GET

        tags: ["friends", "public"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - List[ModelGetUserOutgoingFriendsResponse] (OK)

        400: Bad Request - RestapiErrorResponseV1 (Bad Request)

        401: Unauthorized - RestapiErrorResponseV1 (Unauthorized)

        403: Forbidden - RestapiErrorResponseV1 (Forbidden)

        404: Not Found - RestapiErrorResponseV1 (Not Found)

        500: Internal Server Error - RestapiErrorResponseV1 (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetUserOutgoingFriends.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetUserOutgoingFriendsWithTime)
def get_user_outgoing_friends_with_time(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """get list of outgoing friends with requested time info (getUserOutgoingFriendsWithTime)

    Properties:
        url: /friends/namespaces/{namespace}/me/outgoing-time

        method: GET

        tags: ["friends", "public"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - List[ModelLoadOutgoingFriendsWithTimeResponse] (OK)

        400: Bad Request - RestapiErrorResponseV1 (Bad Request)

        401: Unauthorized - RestapiErrorResponseV1 (Unauthorized)

        403: Forbidden - RestapiErrorResponseV1 (Forbidden)

        404: Not Found - RestapiErrorResponseV1 (Not Found)

        500: Internal Server Error - RestapiErrorResponseV1 (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetUserOutgoingFriendsWithTime.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetUserOutgoingFriendsWithTime)
async def get_user_outgoing_friends_with_time_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """get list of outgoing friends with requested time info (getUserOutgoingFriendsWithTime)

    Properties:
        url: /friends/namespaces/{namespace}/me/outgoing-time

        method: GET

        tags: ["friends", "public"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - List[ModelLoadOutgoingFriendsWithTimeResponse] (OK)

        400: Bad Request - RestapiErrorResponseV1 (Bad Request)

        401: Unauthorized - RestapiErrorResponseV1 (Unauthorized)

        403: Forbidden - RestapiErrorResponseV1 (Forbidden)

        404: Not Found - RestapiErrorResponseV1 (Not Found)

        500: Internal Server Error - RestapiErrorResponseV1 (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetUserOutgoingFriendsWithTime.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UserAcceptFriendRequest)
def user_accept_friend_request(
    body: ModelUserAcceptFriendRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """user accept friend (userAcceptFriendRequest)

    Properties:
        url: /friends/namespaces/{namespace}/me/request/accept

        method: POST

        tags: ["friends", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelUserAcceptFriendRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - RestapiErrorResponseV1 (Bad Request)

        401: Unauthorized - RestapiErrorResponseV1 (Unauthorized)

        403: Forbidden - RestapiErrorResponseV1 (Forbidden)

        404: Not Found - RestapiErrorResponseV1 (Not Found)

        500: Internal Server Error - RestapiErrorResponseV1 (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UserAcceptFriendRequest.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UserAcceptFriendRequest)
async def user_accept_friend_request_async(
    body: ModelUserAcceptFriendRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """user accept friend (userAcceptFriendRequest)

    Properties:
        url: /friends/namespaces/{namespace}/me/request/accept

        method: POST

        tags: ["friends", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelUserAcceptFriendRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - RestapiErrorResponseV1 (Bad Request)

        401: Unauthorized - RestapiErrorResponseV1 (Unauthorized)

        403: Forbidden - RestapiErrorResponseV1 (Forbidden)

        404: Not Found - RestapiErrorResponseV1 (Not Found)

        500: Internal Server Error - RestapiErrorResponseV1 (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UserAcceptFriendRequest.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UserCancelFriendRequest)
def user_cancel_friend_request(
    body: ModelUserCancelFriendRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """user cancel a friend request (userCancelFriendRequest)

    Properties:
        url: /friends/namespaces/{namespace}/me/request/cancel

        method: POST

        tags: ["friends", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelUserCancelFriendRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - RestapiErrorResponseV1 (Bad Request)

        401: Unauthorized - RestapiErrorResponseV1 (Unauthorized)

        403: Forbidden - RestapiErrorResponseV1 (Forbidden)

        404: Not Found - RestapiErrorResponseV1 (Not Found)

        500: Internal Server Error - RestapiErrorResponseV1 (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UserCancelFriendRequest.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UserCancelFriendRequest)
async def user_cancel_friend_request_async(
    body: ModelUserCancelFriendRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """user cancel a friend request (userCancelFriendRequest)

    Properties:
        url: /friends/namespaces/{namespace}/me/request/cancel

        method: POST

        tags: ["friends", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelUserCancelFriendRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - RestapiErrorResponseV1 (Bad Request)

        401: Unauthorized - RestapiErrorResponseV1 (Unauthorized)

        403: Forbidden - RestapiErrorResponseV1 (Forbidden)

        404: Not Found - RestapiErrorResponseV1 (Not Found)

        500: Internal Server Error - RestapiErrorResponseV1 (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UserCancelFriendRequest.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UserGetFriendshipStatus)
def user_get_friendship_status(
    friend_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """user get friendship status (userGetFriendshipStatus)

    Properties:
        url: /friends/namespaces/{namespace}/me/status/{friendId}

        method: GET

        tags: ["friends", "public"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        friend_id: (friendId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelUserGetFriendshipStatusResponse (OK)

        400: Bad Request - RestapiErrorResponseV1 (Bad Request)

        401: Unauthorized - RestapiErrorResponseV1 (Unauthorized)

        403: Forbidden - RestapiErrorResponseV1 (Forbidden)

        500: Internal Server Error - RestapiErrorResponseV1 (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UserGetFriendshipStatus.create(
        friend_id=friend_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UserGetFriendshipStatus)
async def user_get_friendship_status_async(
    friend_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """user get friendship status (userGetFriendshipStatus)

    Properties:
        url: /friends/namespaces/{namespace}/me/status/{friendId}

        method: GET

        tags: ["friends", "public"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        friend_id: (friendId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelUserGetFriendshipStatusResponse (OK)

        400: Bad Request - RestapiErrorResponseV1 (Bad Request)

        401: Unauthorized - RestapiErrorResponseV1 (Unauthorized)

        403: Forbidden - RestapiErrorResponseV1 (Forbidden)

        500: Internal Server Error - RestapiErrorResponseV1 (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UserGetFriendshipStatus.create(
        friend_id=friend_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UserRejectFriendRequest)
def user_reject_friend_request(
    body: ModelUserRejectFriendRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """user reject a friend request (userRejectFriendRequest)

    Properties:
        url: /friends/namespaces/{namespace}/me/request/reject

        method: POST

        tags: ["friends", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelUserRejectFriendRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - RestapiErrorResponseV1 (Bad Request)

        401: Unauthorized - RestapiErrorResponseV1 (Unauthorized)

        403: Forbidden - RestapiErrorResponseV1 (Forbidden)

        404: Not Found - RestapiErrorResponseV1 (Not Found)

        500: Internal Server Error - RestapiErrorResponseV1 (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UserRejectFriendRequest.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UserRejectFriendRequest)
async def user_reject_friend_request_async(
    body: ModelUserRejectFriendRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """user reject a friend request (userRejectFriendRequest)

    Properties:
        url: /friends/namespaces/{namespace}/me/request/reject

        method: POST

        tags: ["friends", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelUserRejectFriendRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - RestapiErrorResponseV1 (Bad Request)

        401: Unauthorized - RestapiErrorResponseV1 (Unauthorized)

        403: Forbidden - RestapiErrorResponseV1 (Forbidden)

        404: Not Found - RestapiErrorResponseV1 (Not Found)

        500: Internal Server Error - RestapiErrorResponseV1 (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UserRejectFriendRequest.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UserRequestFriend)
def user_request_friend(
    body: ModelUserRequestFriendRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """user add friend (userRequestFriend)

    Client should provide either friendID or friendPublicID. If both are provided, friendID will be chosen to be used.

    Properties:
        url: /friends/namespaces/{namespace}/me/request

        method: POST

        tags: ["friends", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelUserRequestFriendRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - (Created)

        400: Bad Request - RestapiErrorResponseV1 (Bad Request)

        401: Unauthorized - RestapiErrorResponseV1 (Unauthorized)

        403: Forbidden - RestapiErrorResponseV1 (Forbidden)

        404: Not Found - RestapiErrorResponseV1 (Not Found)

        422: Unprocessable Entity - RestapiErrorResponseV1 (Unprocessable Entity)

        500: Internal Server Error - RestapiErrorResponseV1 (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UserRequestFriend.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UserRequestFriend)
async def user_request_friend_async(
    body: ModelUserRequestFriendRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """user add friend (userRequestFriend)

    Client should provide either friendID or friendPublicID. If both are provided, friendID will be chosen to be used.

    Properties:
        url: /friends/namespaces/{namespace}/me/request

        method: POST

        tags: ["friends", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelUserRequestFriendRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - (Created)

        400: Bad Request - RestapiErrorResponseV1 (Bad Request)

        401: Unauthorized - RestapiErrorResponseV1 (Unauthorized)

        403: Forbidden - RestapiErrorResponseV1 (Forbidden)

        404: Not Found - RestapiErrorResponseV1 (Not Found)

        422: Unprocessable Entity - RestapiErrorResponseV1 (Unprocessable Entity)

        500: Internal Server Error - RestapiErrorResponseV1 (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UserRequestFriend.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UserUnfriendRequest)
def user_unfriend_request(
    body: ModelUserUnfriendRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """user unfriend a friend (userUnfriendRequest)

    Properties:
        url: /friends/namespaces/{namespace}/me/unfriend

        method: POST

        tags: ["friends", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelUserUnfriendRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - RestapiErrorResponseV1 (Bad Request)

        401: Unauthorized - RestapiErrorResponseV1 (Unauthorized)

        403: Forbidden - RestapiErrorResponseV1 (Forbidden)

        404: Not Found - RestapiErrorResponseV1 (Not Found)

        500: Internal Server Error - RestapiErrorResponseV1 (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UserUnfriendRequest.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UserUnfriendRequest)
async def user_unfriend_request_async(
    body: ModelUserUnfriendRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """user unfriend a friend (userUnfriendRequest)

    Properties:
        url: /friends/namespaces/{namespace}/me/unfriend

        method: POST

        tags: ["friends", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelUserUnfriendRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - RestapiErrorResponseV1 (Bad Request)

        401: Unauthorized - RestapiErrorResponseV1 (Unauthorized)

        403: Forbidden - RestapiErrorResponseV1 (Forbidden)

        404: Not Found - RestapiErrorResponseV1 (Not Found)

        500: Internal Server Error - RestapiErrorResponseV1 (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UserUnfriendRequest.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Iam Service (6.0.1)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.accountcommon_permission_v3 import AccountcommonPermissionV3


class ClientmodelClientUpdateV3Request(Model):
    """Clientmodel client update V3 request (clientmodel.ClientUpdateV3Request)

    Properties:
        client_platform: (clientPlatform) REQUIRED str

        description: (description) REQUIRED str

        audiences: (audiences) OPTIONAL List[str]

        base_uri: (baseUri) OPTIONAL str

        client_name: (clientName) OPTIONAL str

        client_permissions: (clientPermissions) OPTIONAL List[AccountcommonPermissionV3]

        deletable: (deletable) OPTIONAL bool

        namespace: (namespace) OPTIONAL str

        oauth_access_token_expiration: (oauthAccessTokenExpiration) OPTIONAL int

        oauth_access_token_expiration_time_unit: (oauthAccessTokenExpirationTimeUnit) OPTIONAL str

        oauth_refresh_token_expiration: (oauthRefreshTokenExpiration) OPTIONAL int

        oauth_refresh_token_expiration_time_unit: (oauthRefreshTokenExpirationTimeUnit) OPTIONAL str

        redirect_uri: (redirectUri) OPTIONAL str

        scopes: (scopes) OPTIONAL List[str]

        two_factor_enabled: (twoFactorEnabled) OPTIONAL bool
    """

    # region fields

    client_platform: str  # REQUIRED
    description: str  # REQUIRED
    audiences: List[str]  # OPTIONAL
    base_uri: str  # OPTIONAL
    client_name: str  # OPTIONAL
    client_permissions: List[AccountcommonPermissionV3]  # OPTIONAL
    deletable: bool  # OPTIONAL
    namespace: str  # OPTIONAL
    oauth_access_token_expiration: int  # OPTIONAL
    oauth_access_token_expiration_time_unit: str  # OPTIONAL
    oauth_refresh_token_expiration: int  # OPTIONAL
    oauth_refresh_token_expiration_time_unit: str  # OPTIONAL
    redirect_uri: str  # OPTIONAL
    scopes: List[str]  # OPTIONAL
    two_factor_enabled: bool  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_client_platform(self, value: str) -> ClientmodelClientUpdateV3Request:
        self.client_platform = value
        return self

    def with_description(self, value: str) -> ClientmodelClientUpdateV3Request:
        self.description = value
        return self

    def with_audiences(self, value: List[str]) -> ClientmodelClientUpdateV3Request:
        self.audiences = value
        return self

    def with_base_uri(self, value: str) -> ClientmodelClientUpdateV3Request:
        self.base_uri = value
        return self

    def with_client_name(self, value: str) -> ClientmodelClientUpdateV3Request:
        self.client_name = value
        return self

    def with_client_permissions(
        self, value: List[AccountcommonPermissionV3]
    ) -> ClientmodelClientUpdateV3Request:
        self.client_permissions = value
        return self

    def with_deletable(self, value: bool) -> ClientmodelClientUpdateV3Request:
        self.deletable = value
        return self

    def with_namespace(self, value: str) -> ClientmodelClientUpdateV3Request:
        self.namespace = value
        return self

    def with_oauth_access_token_expiration(
        self, value: int
    ) -> ClientmodelClientUpdateV3Request:
        self.oauth_access_token_expiration = value
        return self

    def with_oauth_access_token_expiration_time_unit(
        self, value: str
    ) -> ClientmodelClientUpdateV3Request:
        self.oauth_access_token_expiration_time_unit = value
        return self

    def with_oauth_refresh_token_expiration(
        self, value: int
    ) -> ClientmodelClientUpdateV3Request:
        self.oauth_refresh_token_expiration = value
        return self

    def with_oauth_refresh_token_expiration_time_unit(
        self, value: str
    ) -> ClientmodelClientUpdateV3Request:
        self.oauth_refresh_token_expiration_time_unit = value
        return self

    def with_redirect_uri(self, value: str) -> ClientmodelClientUpdateV3Request:
        self.redirect_uri = value
        return self

    def with_scopes(self, value: List[str]) -> ClientmodelClientUpdateV3Request:
        self.scopes = value
        return self

    def with_two_factor_enabled(self, value: bool) -> ClientmodelClientUpdateV3Request:
        self.two_factor_enabled = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "client_platform"):
            result["clientPlatform"] = str(self.client_platform)
        elif include_empty:
            result["clientPlatform"] = ""
        if hasattr(self, "description"):
            result["description"] = str(self.description)
        elif include_empty:
            result["description"] = ""
        if hasattr(self, "audiences"):
            result["audiences"] = [str(i0) for i0 in self.audiences]
        elif include_empty:
            result["audiences"] = []
        if hasattr(self, "base_uri"):
            result["baseUri"] = str(self.base_uri)
        elif include_empty:
            result["baseUri"] = ""
        if hasattr(self, "client_name"):
            result["clientName"] = str(self.client_name)
        elif include_empty:
            result["clientName"] = ""
        if hasattr(self, "client_permissions"):
            result["clientPermissions"] = [
                i0.to_dict(include_empty=include_empty)
                for i0 in self.client_permissions
            ]
        elif include_empty:
            result["clientPermissions"] = []
        if hasattr(self, "deletable"):
            result["deletable"] = bool(self.deletable)
        elif include_empty:
            result["deletable"] = False
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "oauth_access_token_expiration"):
            result["oauthAccessTokenExpiration"] = int(
                self.oauth_access_token_expiration
            )
        elif include_empty:
            result["oauthAccessTokenExpiration"] = 0
        if hasattr(self, "oauth_access_token_expiration_time_unit"):
            result["oauthAccessTokenExpirationTimeUnit"] = str(
                self.oauth_access_token_expiration_time_unit
            )
        elif include_empty:
            result["oauthAccessTokenExpirationTimeUnit"] = ""
        if hasattr(self, "oauth_refresh_token_expiration"):
            result["oauthRefreshTokenExpiration"] = int(
                self.oauth_refresh_token_expiration
            )
        elif include_empty:
            result["oauthRefreshTokenExpiration"] = 0
        if hasattr(self, "oauth_refresh_token_expiration_time_unit"):
            result["oauthRefreshTokenExpirationTimeUnit"] = str(
                self.oauth_refresh_token_expiration_time_unit
            )
        elif include_empty:
            result["oauthRefreshTokenExpirationTimeUnit"] = ""
        if hasattr(self, "redirect_uri"):
            result["redirectUri"] = str(self.redirect_uri)
        elif include_empty:
            result["redirectUri"] = ""
        if hasattr(self, "scopes"):
            result["scopes"] = [str(i0) for i0 in self.scopes]
        elif include_empty:
            result["scopes"] = []
        if hasattr(self, "two_factor_enabled"):
            result["twoFactorEnabled"] = bool(self.two_factor_enabled)
        elif include_empty:
            result["twoFactorEnabled"] = False
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        client_platform: str,
        description: str,
        audiences: Optional[List[str]] = None,
        base_uri: Optional[str] = None,
        client_name: Optional[str] = None,
        client_permissions: Optional[List[AccountcommonPermissionV3]] = None,
        deletable: Optional[bool] = None,
        namespace: Optional[str] = None,
        oauth_access_token_expiration: Optional[int] = None,
        oauth_access_token_expiration_time_unit: Optional[str] = None,
        oauth_refresh_token_expiration: Optional[int] = None,
        oauth_refresh_token_expiration_time_unit: Optional[str] = None,
        redirect_uri: Optional[str] = None,
        scopes: Optional[List[str]] = None,
        two_factor_enabled: Optional[bool] = None,
        **kwargs,
    ) -> ClientmodelClientUpdateV3Request:
        instance = cls()
        instance.client_platform = client_platform
        instance.description = description
        if audiences is not None:
            instance.audiences = audiences
        if base_uri is not None:
            instance.base_uri = base_uri
        if client_name is not None:
            instance.client_name = client_name
        if client_permissions is not None:
            instance.client_permissions = client_permissions
        if deletable is not None:
            instance.deletable = deletable
        if namespace is not None:
            instance.namespace = namespace
        if oauth_access_token_expiration is not None:
            instance.oauth_access_token_expiration = oauth_access_token_expiration
        if oauth_access_token_expiration_time_unit is not None:
            instance.oauth_access_token_expiration_time_unit = (
                oauth_access_token_expiration_time_unit
            )
        if oauth_refresh_token_expiration is not None:
            instance.oauth_refresh_token_expiration = oauth_refresh_token_expiration
        if oauth_refresh_token_expiration_time_unit is not None:
            instance.oauth_refresh_token_expiration_time_unit = (
                oauth_refresh_token_expiration_time_unit
            )
        if redirect_uri is not None:
            instance.redirect_uri = redirect_uri
        if scopes is not None:
            instance.scopes = scopes
        if two_factor_enabled is not None:
            instance.two_factor_enabled = two_factor_enabled
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ClientmodelClientUpdateV3Request:
        instance = cls()
        if not dict_:
            return instance
        if "clientPlatform" in dict_ and dict_["clientPlatform"] is not None:
            instance.client_platform = str(dict_["clientPlatform"])
        elif include_empty:
            instance.client_platform = ""
        if "description" in dict_ and dict_["description"] is not None:
            instance.description = str(dict_["description"])
        elif include_empty:
            instance.description = ""
        if "audiences" in dict_ and dict_["audiences"] is not None:
            instance.audiences = [str(i0) for i0 in dict_["audiences"]]
        elif include_empty:
            instance.audiences = []
        if "baseUri" in dict_ and dict_["baseUri"] is not None:
            instance.base_uri = str(dict_["baseUri"])
        elif include_empty:
            instance.base_uri = ""
        if "clientName" in dict_ and dict_["clientName"] is not None:
            instance.client_name = str(dict_["clientName"])
        elif include_empty:
            instance.client_name = ""
        if "clientPermissions" in dict_ and dict_["clientPermissions"] is not None:
            instance.client_permissions = [
                AccountcommonPermissionV3.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["clientPermissions"]
            ]
        elif include_empty:
            instance.client_permissions = []
        if "deletable" in dict_ and dict_["deletable"] is not None:
            instance.deletable = bool(dict_["deletable"])
        elif include_empty:
            instance.deletable = False
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if (
            "oauthAccessTokenExpiration" in dict_
            and dict_["oauthAccessTokenExpiration"] is not None
        ):
            instance.oauth_access_token_expiration = int(
                dict_["oauthAccessTokenExpiration"]
            )
        elif include_empty:
            instance.oauth_access_token_expiration = 0
        if (
            "oauthAccessTokenExpirationTimeUnit" in dict_
            and dict_["oauthAccessTokenExpirationTimeUnit"] is not None
        ):
            instance.oauth_access_token_expiration_time_unit = str(
                dict_["oauthAccessTokenExpirationTimeUnit"]
            )
        elif include_empty:
            instance.oauth_access_token_expiration_time_unit = ""
        if (
            "oauthRefreshTokenExpiration" in dict_
            and dict_["oauthRefreshTokenExpiration"] is not None
        ):
            instance.oauth_refresh_token_expiration = int(
                dict_["oauthRefreshTokenExpiration"]
            )
        elif include_empty:
            instance.oauth_refresh_token_expiration = 0
        if (
            "oauthRefreshTokenExpirationTimeUnit" in dict_
            and dict_["oauthRefreshTokenExpirationTimeUnit"] is not None
        ):
            instance.oauth_refresh_token_expiration_time_unit = str(
                dict_["oauthRefreshTokenExpirationTimeUnit"]
            )
        elif include_empty:
            instance.oauth_refresh_token_expiration_time_unit = ""
        if "redirectUri" in dict_ and dict_["redirectUri"] is not None:
            instance.redirect_uri = str(dict_["redirectUri"])
        elif include_empty:
            instance.redirect_uri = ""
        if "scopes" in dict_ and dict_["scopes"] is not None:
            instance.scopes = [str(i0) for i0 in dict_["scopes"]]
        elif include_empty:
            instance.scopes = []
        if "twoFactorEnabled" in dict_ and dict_["twoFactorEnabled"] is not None:
            instance.two_factor_enabled = bool(dict_["twoFactorEnabled"])
        elif include_empty:
            instance.two_factor_enabled = False
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ClientmodelClientUpdateV3Request]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ClientmodelClientUpdateV3Request]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ClientmodelClientUpdateV3Request,
        List[ClientmodelClientUpdateV3Request],
        Dict[Any, ClientmodelClientUpdateV3Request],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "clientPlatform": "client_platform",
            "description": "description",
            "audiences": "audiences",
            "baseUri": "base_uri",
            "clientName": "client_name",
            "clientPermissions": "client_permissions",
            "deletable": "deletable",
            "namespace": "namespace",
            "oauthAccessTokenExpiration": "oauth_access_token_expiration",
            "oauthAccessTokenExpirationTimeUnit": "oauth_access_token_expiration_time_unit",
            "oauthRefreshTokenExpiration": "oauth_refresh_token_expiration",
            "oauthRefreshTokenExpirationTimeUnit": "oauth_refresh_token_expiration_time_unit",
            "redirectUri": "redirect_uri",
            "scopes": "scopes",
            "twoFactorEnabled": "two_factor_enabled",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "clientPlatform": True,
            "description": True,
            "audiences": False,
            "baseUri": False,
            "clientName": False,
            "clientPermissions": False,
            "deletable": False,
            "namespace": False,
            "oauthAccessTokenExpiration": False,
            "oauthAccessTokenExpirationTimeUnit": False,
            "oauthRefreshTokenExpiration": False,
            "oauthRefreshTokenExpirationTimeUnit": False,
            "redirectUri": False,
            "scopes": False,
            "twoFactorEnabled": False,
        }

    # endregion static methods
